/********************************************************************
   T/SDAS-AD1/14/300K-DA1/16/300K DCM rev.2A Demo Utility for C3x DSP. rev.1A
   (C) MicroLAB Systems, 1999-2001


   Description:
   ------------
	1. This demo writes SINE to DAC channelin ASYNCHRONOUS mode.
    2. DSP on-chip timer TM-0 is used for sampling frequency.
	3. DSP on-chip timer TM-1 is used to set ASYNC mode in case DCM is
       configured in S/W PROGRAMMABLE OPERATION MODE, otherwise DCM
	   must be configured in ASYNC ONLY mode.

   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-3x (except TORNADO-32L), TORNADO-P3x 
	and TORNADO-E3x	boards and actually differs in PIOX/SIOX reset control 
	and	initialization of C3x DSP on-chip timer in order to meet DSP cycle 
	time. This demo does not utilize any host PC ISA/PCI resources for 
	TORNADO-3x/P3x boards and can run in stand-alone mode. MIRAGE JTAG 
	emulator and TI CCS tools are required to recompile this demo and to 
	upload code to the DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and is provided as it is 
        without any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
		  - either ASYNCHRONOUS ONLY mode or S/W PROGRAMMABLE OPERATION MODE 
		    must be selected
		  - XTIMER-1 line must be used to set operation mode if S/W PROGRAMMABLE
		    OPERATION MODE has been selected
	  	  - C3x SIO-0 port is used 

***********************************************************************/

#define	_E3X_		1
#define	_T3X_		2
#define	_TP3X_		3

#define	_C3X_SIO0_	0
#define	_C3X_SIO1_	1

//~~~~~~~~~ TORNADO and C3x SIO port selectors for compilation ~~~~~~~
//!!!! modify this def in order to compile for specific TORNADO platform
#define	_TORNADO_	_T3X_

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define	_E3X_SELECTED	(_TORNADO_==_E3X_)
#define	_T3X_SELECTED	(_TORNADO_==_T3X_)
#define	_TP3X_SELECTED	(_TORNADO_==_TP3X_)


#include	"float.h"
#include	"math.h"
#include	"c3x_def.h"
#include	"sxa1_dsp.h"

//------- macros for SIO-0 port access ----------------------
#define	_WAIT_SIO_TRDY		while(!((*C3X_SIO0_GC) & 0x2))
#define	_WAIT_SIO_RRDY		while(!((*C3X_SIO0_GC) & 0x1))
#define	_WRITE_SIO_DATA		(*C3X_SIO0_XD)
#define _READ_SIO_DATA		(*C3X_SIO0_RD)


//------ TORNADO platform specific fucntions --------------------
#if(_E3X_SELECTED)
#define _REMOVE_SIOX_RESET	TE3X_set_pxsx_run(0x3)
#include	"te3x_dsp.h"
#endif

#if(_T3X_SELECTED)
// SIOX reset control is not available for T3x boards.
// In case T33 board is installed, disable PIOX/SIOX reset control via
// host PC interface.
#define _REMOVE_SIOX_RESET	;
#include	"t3x_dsp.h"
#endif

#if(_TP3X_SELECTED)
#define _REMOVE_SIOX_RESET	TP3X_set_pxsx_run(0x7)
#include	"tp3x_dsp.h"
#endif
//--------------------------------------------------------------



	//--- write COMMAND DATA WORD in ASYNC mode
void write_async_cmnd(data)
unsigned data;
{
		//-- wait for SIO-0 transmitter is ready
	_WAIT_SIO_TRDY;

		//-- write command
	_WRITE_SIO_DATA = data;
}



	//--- read ADC DATA
int read_adc()
{ unsigned rdr;

		//-- wait for receiver is ready and read data
	_WAIT_SIO_RRDY;
	rdr=_READ_SIO_DATA;
    return(((rdr & TSDAS_ADDA1_ADC_DATA_BITMASK)<<16) >>16);
}



void init_cpu()
 {
	*C3X_PBUS_GC = 0x700;
	C3X_CACHE_ENABLE;
 }



#define	_TM_CONTROL_RG_DATA_OUT0	0x00000002  // TM as OUT pin: '0' output state
#define	_TM_CONTROL_RG_DATA_OUT1	0x00000006  // TM as OUT pin: '1' output state

void init_sio_async()
{
		//-- configure C3x SIO port
	*C3X_SIO0_GC 	= 	TSDAS_ADDA1_C3X_SIO_GCRG_DATA_ASYNC;
	*C3X_SIO0_XC 	= 	TSDAS_ADDA1_C3X_SIO_XCRG_DATA;
	*C3X_SIO0_RC 	= 	TSDAS_ADDA1_C3X_SIO_RCRG_DATA;
	*C3X_SIO0_GC 	|= 	TSDAS_ADDA1_C3X_SIO_GCRG_GO_MASK;

		//-- configure TM-1 as OUTPUT pin and set it to '0' in order to set ASYNC mode
	*C3X_TM1_GC  	= _TM_CONTROL_RG_DATA_OUT0;	

		//-- release SIOX reset control and enable DCM
	_REMOVE_SIOX_RESET;
}



	//-- configure TM-0 as sampling frequency generator

#define	_TM_CONTROL_RG_DATA_COUNT	0x00000301  // TM as CLOCK output: C/P=clock, CLKSRC=internal, FUNC=timer
#define	_TM_CONTROL_RG_RUN			0x000000c0

#define _SAMPLING_FREQ				10000		// desired sampling freq (Hz)
#define _C3X_TIMER_SRCCLK_FREQ		15000000	// C30/C31 on-chip TM-0/1 internal src clk (Hz)

void set_fs()
{
		//-- configure TM-0 for sampling frequency
	*C3X_TM0_GC 	= _TM_CONTROL_RG_DATA_COUNT;	
	*C3X_TM0_CNT 	= 0;	
	*C3X_TM0_PER  	= (_C3X_TIMER_SRCCLK_FREQ/(2*_SAMPLING_FREQ));	

		//-- now start TM-0 timer
	*C3X_TM0_GC |= _TM_CONTROL_RG_RUN;
}


	//-- generate sampling frequency via TIMER-0
void	wait_fs()
{
	//-- wait until TM0 output is '0'
    while(((*C3X_TM0_GC) & 0x800)!=0);

	//-- wait until TM0 output is '1'
    while(((*C3X_TM0_GC) & 0x800)==0);
}



#define	_EPOCH_LENGTH	1000
#define	TWO_PI			(2.0*3.1415926)

	//-- DACs data arrays
int dac_data[_EPOCH_LENGTH];



void main()
{  int i;

	init_cpu();
	init_sio_async();
	set_fs();		

		//-- fill in DAC array with SINE data
	for(i=0;i<_EPOCH_LENGTH;i++)
		   dac_data[i]=  (int) ((float) (32000.0 * sin( 5.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));


    i=0;
	while (1)
	 {
		//-- wait for sampling freq event
		wait_fs();	

       	//-- load DAC
	    write_async_cmnd(  TSDAS_ADDA1_DAC_WR_CMND
					     | (dac_data[i] & TSDAS_ADDA1_DAC_DATA_BITMASK)
					    );
		i++;
	  	i %= _EPOCH_LENGTH;
	 }
}

