/********************************************************************
   T/SDAS-AD1/14/300K-DA1/16/300K DCM rev.2A Demo Utility for C6x/C64xx DSP. rev.1A-1
   (C) MicroLAB Systems, 1999-2005


   Description:
   ------------
	1. This demo writes SINE to DAC channelin ASYNCHRONOUS mode.
    2. DSP on-chip timer TM-0 is used for sampling frequency.
	3. DSP on-chip timer TM-1 is used to set ASYNC mode in case DCM is
       configured in S/W PROGRAMMABLE OPERATION MODE, otherwise DCM
	   must be configured in ASYNC ONLY mode.

   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x/P64xx and TORNADO-E6x/E64xx
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. MIRAGE JTAG emulator and TI CCS tools are
	required to recompile this demo and to upload code to the
	DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and is provided as it is 
        without any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
     1. DCM on-board configuration switch:
		  - either ASYNCHRONOUS ONLY mode or S/W PROGRAMMABLE OPERATION MODE 
		    must be selected
		  - XTIMER-1 line must be used to set operation mode if S/W PROGRAMMABLE
		    OPERATION MODE has been selected
	  	  - C6x McBSP-0 port is used (change this via _C6X_MCBSP_ def)
			
     2. TORNADO on-board configuration jumpers:
		  - TM0-OUT and TM1-OUT jumpers shall be installed
***********************************************************************/

#define	_E6X_		1
#define	_E64XX_		4
#define	_T6X_		2
#define	_TP6X_		3
#define	_TP64XX_	5

#define	_C6X_MCBSP0_	0
#define	_C6X_MCBSP1_	1

//~~~~~~~~~ TORNADO and C6x McBSP port selectors for compilation ~~~~~~~
//!!!! modify this def in order to compile for specific TORNADO platform
#define	_TORNADO_	_E64XX_

//!!!! modify this def in order to compile for McBSP-0 or McBSP-1 port
#define	_C6X_MCBSP_	_C6X_MCBSP0_
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define	_E6X_SELECTED		(_TORNADO_==_E6X_)
#define	_E64XX_SELECTED		(_TORNADO_==_E64XX_)
#define	_T6X_SELECTED		(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED		(_TORNADO_==_TP6X_)
#define	_TP64XX_SELECTED	(_TORNADO_==_TP64XX_)


#define	_C6X_MCBSP0_SELECTED	(_C6X_MCBSP_==_C6X_MCBSP0_)
#define	_C6X_MCBSP1_SELECTED	(_C6X_MCBSP_==_C6X_MCBSP1_)

#if(_E64XX_SELECTED)
#define	__TE6X_SELECT_E64XX__	1
#endif

#if(_TP64XX_SELECTED)
#define	__TP6X_SELECT_TP64XX__	1
#endif

#include	"math.h"
#include	"c6x_def.h"
#include	"sxa1_dsp.h"

//------- macros for McBSP port access ----------------------
#if(_C6X_MCBSP0_SELECTED)	// define for C6x McBSP-0 port
#define	_WAIT_SIO_TRDY		while(!((*C6X_MCBSP0_SPCR) & 0x20000))
#define	_WAIT_SIO_RRDY		while(!((*C6X_MCBSP0_SPCR) & 0x2))
#define	_WRITE_SIO_DATA		(*C6X_MCBSP0_DXR)
#define _READ_SIO_DATA		(*C6X_MCBSP0_DRR)
#endif

#if(_C6X_MCBSP1_SELECTED)	// define for C6x McBSP-1 port
#define	_WAIT_SIO_TRDY		while(!((*C6X_MCBSP1_SPCR) & 0x20000))
#define	_WAIT_SIO_RRDY		while(!((*C6X_MCBSP1_SPCR) & 0x2))
#define	_WRITE_SIO_DATA		(*C6X_MCBSP1_DXR)
#define _READ_SIO_DATA		(*C6X_MCBSP1_DRR)
#endif


//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED || _E64XX_SELECTED)
#define _REMOVE_SIOX_RESET	TE6X_set_pxsx_run(0x3)
#include	"te6x_dsp.h"
#endif

#if(_T6X_SELECTED)
#define _REMOVE_SIOX_RESET	T6X_set_pxsx_run(0x7)
#include	"t6x_dsp.h"
#endif

#if(_TP6X_SELECTED || _TP64XX_SELECTED)
#define _REMOVE_SIOX_RESET	TP6X_set_pxsx_run(0x7)
#include	"tp6x_dsp.h"
#endif
//--------------------------------------------------------------


	//--- write COMMAND DATA WORD in ASYNC mode
void write_async_cmnd(data)
unsigned data;
{
		//-- wait for SIO-0 transmitter is ready
	_WAIT_SIO_TRDY;

		//-- write command
	_WRITE_SIO_DATA = data;
}



void init_cpu()
{
	// perform TORNADO-E6x/E64xx DSP System initialization
#if(_E6X_SELECTED || _E64XX_SELECTED)

		// init DSP EMIF
	TE6X_init_dsp_emif(TE6X_XMEM_SBSRAM);
#endif




	// perform TORNADO-6x DSP System initialization
#if(_T6X_SELECTED)

		// init DSP EMIF
	*C6X_EMIF_GCR=T6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=T6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=T6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=T6X_EMIF_CE2_DATA;
	*C6X_EMIF_CE3=T6X_EMIF_CE3_DATA;
#endif


	// perform TORNADO-P6x DSP System initialization
#if(_TP6X_SELECTED)

		// init DSP EMIF
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);
#endif


	// perform TORNADO-P64xx DSP System initialization
#if(_TP64XX_SELECTED)

		// init DSP EMIF
	TP6X_init_dsp_emif();
#endif
}



#define	_TM_CONTROL_RG_DATA_OUT0	0x00000000  // TM as OUT pin: '0' output state
#define	_TM_CONTROL_RG_DATA_OUT1	0x00000004  // TM as OUT pin: '1' output state


void init_sio_async()
{
		//-- configure C6x McBSP port for ASYNC mode
#if(_C6X_MCBSP0_SELECTED)	// define for C6x McBSP-0 port
	*C6X_MCBSP0_SPCR = TSDAS_ADDA1_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP0_PCR  = TSDAS_ADDA1_C6X_MCBSP_PCR_DATA_ASYNC;
	*C6X_MCBSP0_XCR  = TSDAS_ADDA1_C6X_MCBSP_XCR_DATA_ASYNC;
	*C6X_MCBSP0_RCR  = TSDAS_ADDA1_C6X_MCBSP_RCR_DATA_ASYNC;
	*C6X_MCBSP0_SRGR = TSDAS_ADDA1_C6201_MCBSP_SRGR_DATA;
	*C6X_MCBSP0_SPCR |= TSDAS_ADDA1_C6X_MCBSP_SPCR_GO_MASK;
#endif

#if(_C6X_MCBSP1_SELECTED)	// define for C6x McBSP-1 port
	*C6X_MCBSP1_SPCR = TSDAS_ADDA1_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP1_PCR  = TSDAS_ADDA1_C6X_MCBSP_PCR_DATA_ASYNC;
	*C6X_MCBSP1_XCR  = TSDAS_ADDA1_C6X_MCBSP_XCR_DATA_ASYNC;
	*C6X_MCBSP1_RCR  = TSDAS_ADDA1_C6X_MCBSP_RCR_DATA_ASYNC;
	*C6X_MCBSP1_SRGR = TSDAS_ADDA1_C6201_MCBSP_SRGR_DATA;
	*C6X_MCBSP1_SPCR |= TSDAS_ADDA1_C6X_MCBSP_SPCR_GO_MASK;
#endif

		//-- configure TM-1 as OUTPUT pin and set it to '0' in order to set ASYNC mode
	*C6X_TM1_CONTROL   	= _TM_CONTROL_RG_DATA_OUT0;	

		//-- release SIOX reset control and enable DCM
	_REMOVE_SIOX_RESET;
}


	//-- configure TM-0 as sampling frequency generator
#define	_TM_CONTROL_RG_DATA_COUNT	0x00000301         // TM as CLOCK output: C/P=clock, CLKSRC=internal, FUNC=timer
#define	_TM_CONTROL_RG_RUN		0x000000c0

#define _SAMPLING_FREQ			100000		   // desired sampling freq (Hz)
#if(_E64XX_SELECTED || _TP64XX_SELECTED)
	#define _C6X_TIMER_SRCCLK_FREQ		125000000	// TM-0 internal src clk (Hz) for C64xx (DSP_CLK/8)
#else
	#define _C6X_TIMER_SRCCLK_FREQ		75000000	// TM-0 internal src clk (Hz) for C6x (DSP_CLK/4)
#endif

void set_fs()
{
		//-- configure TM-0 for sampling frequency
	*C6X_TM0_CONTROL = _TM_CONTROL_RG_DATA_COUNT;	
	*C6X_TM0_COUNTER = 0;	
	*C6X_TM0_PERIOD  = (_C6X_TIMER_SRCCLK_FREQ/(2*_SAMPLING_FREQ));	

		//-- now start TM-0 timer
	*C6X_TM0_CONTROL |= _TM_CONTROL_RG_RUN;
}



	//-- generates sampling frequency via TIMER-0
void	wait_fs()
{
	//-- wait for TM0 output is '0'
    while(((*C6X_TM0_CONTROL) & 0x800)!=0);

	//-- wait for TM0 output is '1'
    while(((*C6X_TM0_CONTROL) & 0x800)==0);
}



#define	_EPOCH_LENGTH	1000
#define	TWO_PI          (2.0*3.1415926)

	//-- DACs data arrays
int dac_data[_EPOCH_LENGTH];



void main()
{  int i;

	init_cpu();
	init_sio_async();
	set_fs();		

		//-- fill in DAC array with SINE data
	for(i=0;i<_EPOCH_LENGTH;i++)
		   dac_data[i]=  (int) ((float) (32000.0 * sin(5.0*TWO_PI*((float)i)/((float) _EPOCH_LENGTH))));


    i=0;
	while (1)
	 {
		//-- wait for sampling freq event
		wait_fs();	

       	//-- load DAC
	    write_async_cmnd(  TSDAS_ADDA1_DAC_WR_CMND
					     | (dac_data[i] & TSDAS_ADDA1_DAC_DATA_BITMASK)
					    );
		i++;
	  	i %= _EPOCH_LENGTH;
	 }
}