/********************************************************************
   T/SDAS-AD1/14/300K-DA1/16/300K DCM rev.2A Demo Utility for C3x DSP. rev.1A
   (C) MicroLAB Systems, 1999-2001


   Description:
   ------------
	1. This demo writes provides pass-through for A/D-D/A channel in SYNC mode.
    2. DCM on-board PFG is used to set sampling frequency.
	3. DSP on-chip timer TM-1 is used to set SYNC mode, and on-board configuration
	   swict must be set to S/W PROGRAMMABLE OPERATION MODE and XTIMER-1 line to
	   be used to set s/w programmable SYNC mode.

   TORNADO platform:
   -----------------
    This demo compiles for TORNADO-3x (except TORNADO-32L), TORNADO-P3x 
	and TORNADO-E3x	boards and actually differs in PIOX/SIOX reset control 
	and	initialization of C3x DSP on-chip timer in order to meet DSP cycle 
	time. This demo does not utilize any host PC ISA/PCI resources for 
	TORNADO-3x/P3x boards and can run in stand-alone mode. MIRAGE JTAG 
	emulator and TI CCS tools are required to recompile this demo and to 
	upload code to the DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and is provided as it is 
    without any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
		  - S/W PROGRAMMABLE OPERATION MODE must be selected
		  - XTIMER-1 line must be used to set operation mode 
	  	  - C3x SIO-0 port is used 

***********************************************************************/

#define	_E3X_		1
#define	_T3X_		2
#define	_TP3X_		3

#define	_C3X_SIO0_	0
#define	_C3X_SIO1_	1

//~~~~~~~~~ TORNADO and C3x SIO port selectors for compilation ~~~~~~~
//!!!! modify this def in order to compile for specific TORNADO platform
#define	_TORNADO_	_T3X_

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define	_E3X_SELECTED	(_TORNADO_==_E3X_)
#define	_T3X_SELECTED	(_TORNADO_==_T3X_)
#define	_TP3X_SELECTED	(_TORNADO_==_TP3X_)


#include	"math.h"
#include	"c3x_def.h"
#include	"sxa1_dsp.h"

//-- macros for SIO-0 port access
#define	_WAIT_SIO_TRDY		while(!((*C3X_SIO0_GC) & 0x2))
#define	_WAIT_SIO_TEMPTY	while( ((*C3X_SIO0_GC) & 0x8))
#define	_WAIT_SIO_RRDY		while(!((*C3X_SIO0_GC) & 0x1))
#define	_WRITE_SIO_DATA		(*C3X_SIO0_XD)
#define _READ_SIO_DATA		(*C3X_SIO0_RD)


//------ TORNADO platform specific fucntions --------------------
#if(_E3X_SELECTED)
#define _REMOVE_SIOX_RESET	TE3X_set_pxsx_run(0x3)
#include	"te3x_dsp.h"
#endif

#if(_T3X_SELECTED)
// SIOX reset control is not available for T3x boards.
// In case T33 board is installed, disable PIOX/SIOX reset control via
// host PC interface.
#define _REMOVE_SIOX_RESET	;
#include	"t3x_dsp.h"
#endif

#if(_TP3X_SELECTED)
#define _REMOVE_SIOX_RESET	TP3X_set_pxsx_run(0x7)
#include	"tp3x_dsp.h"
#endif
//--------------------------------------------------------------


		/* X/R interrupt handler parameters	*/

int	*dac_data_start_ptr,
    *dac_get_ptr,
	dac_data_count,
    dac_data_count_max;

int	*adc_save_ptr,
	*adc_data_start_ptr,
	adc_data_count,
	adc_data_count_max ;



	//-- SIO Transmitter interrupt.
	//   This IRQ is generated by after the transmitter gets empty after
	//   DCM has generated FSX and initialized transmission of DAC data.

interrupt void c_int01()
{
	//   write data to DAC from DATA array, which is filled in with 
	//   real-time ADC data.

	_WRITE_SIO_DATA = *dac_get_ptr;
    dac_get_ptr++;

    if((++dac_data_count) >=dac_data_count_max)
	 {       
		dac_data_count=0;
	    dac_get_ptr=dac_data_start_ptr;
	 }

}


	//-- Receiver interrupt.
	//   Read and save ADC data.
int r_data;

interrupt void c_int10()
{
    r_data = _READ_SIO_DATA;

		// save ADC data in DATA array in particular ADC# subarray
    *adc_save_ptr = ((r_data <<16 )>>16);
    adc_save_ptr++;
    if((++adc_data_count) >=adc_data_count_max)
	 { 
		adc_data_count=0;
       	adc_save_ptr=adc_data_start_ptr;
	 }
}


void init_cpu()
 {
	*C3X_PBUS_GC = 0x700;
	C3X_CACHE_ENABLE;
 }



void init_sio_async()
{
		//-- configure C3x SIO port for ASYNC mode
	*C3X_SIO0_GC 	= 	TSDAS_ADDA1_C3X_SIO_GCRG_DATA_ASYNC;
	*C3X_SIO0_XC 	= 	TSDAS_ADDA1_C3X_SIO_XCRG_DATA;
	*C3X_SIO0_RC 	= 	TSDAS_ADDA1_C3X_SIO_RCRG_DATA;
	*C3X_SIO0_GC 	|= 	TSDAS_ADDA1_C3X_SIO_GCRG_GO_MASK;
}



void init_sio_sync()
{
		//-- configure C3x SIO port for SYNC mode with RINT/XINT
	*C3X_SIO0_GC 	= 	(TSDAS_ADDA1_C3X_SIO_GCRG_DATA_SYNC | TSDAS_ADDA1_C3X_SIO_GCRG_RINT_MASK | TSDAS_ADDA1_C3X_SIO_GCRG_XINT_MASK);
	*C3X_SIO0_XC 	= 	TSDAS_ADDA1_C3X_SIO_XCRG_DATA;
	*C3X_SIO0_RC 	= 	TSDAS_ADDA1_C3X_SIO_RCRG_DATA;

	// start R/X
	*C3X_SIO0_GC 	|= 	TSDAS_ADDA1_C3X_SIO_GCRG_GO_MASK;

	// write false data to transmitter shift register in order 
	// these can be shifted out on he 1st FSX and prior interrupt occur
	_WRITE_SIO_DATA = 0;

}




	//--- write COMMAND DATA WORD in ASYNC mode
void write_async_cmnd(data)
unsigned data;
{
		//-- wait for SIO-0 transmitter is ready
	_WAIT_SIO_TRDY;

		//-- write command
	_WRITE_SIO_DATA = data;
}


	//-- configure TM-1 as OUTPUT pin for s/w programmable mode control
#define	_TM_CONTROL_RG_DATA_OUT0	0x00000002  // TM as OUT pin: '0' output state
#define	_TM_CONTROL_RG_DATA_OUT1	0x00000006  // TM as OUT pin: '1' output state

void set_async_mode()
{
		//-- configure TM-1 as OUTPUT pin
	*C3X_TM1_GC  	= _TM_CONTROL_RG_DATA_OUT0;	
}

void set_sync_mode()
{
		//-- configure TM-1 as OUTPUT pin
	*C3X_TM1_GC  	= _TM_CONTROL_RG_DATA_OUT1;	
}



#define	_EPOCH_LENGTH	20

// ADC/DAC data array
int data[_EPOCH_LENGTH];


// PFG programming
	// PFG_out=(10.0MHz*N*V)/(M*R*X*Z)
	// set PFG = 50 kHz 
	// (N=32, M=25, V=8, X=1, R=8, Z=256)
#define PFG2_N_DATA       32
#define PFG2_M_DATA       25
#define PFG2_V_DATA       TSDAS_ADDA1_PFG_V_X8
#define PFG2_X_DATA       TSDAS_ADDA1_PFG_X_DIV1
#define PFG2_R_DATA       TSDAS_ADDA1_PFG_R_DIV8
#define PFG2_Z_DATA       TSDAS_ADDA1_PFG_Z_DIV256



void main()
{  
	init_cpu();

	// first set ASYNC mode in order to program Fs SELECTOR and PFG
	set_async_mode();
	init_sio_async();

	// release SIOX reset control and enable DCM
	_REMOVE_SIOX_RESET;

	// set Fs selector to on-board PFG
	write_async_cmnd(TSDAS_ADDA1_CNTR_RG_WR_CMND | TSDAS_ADDA1_FSSEL_PFG);

	// configure PFG
	write_async_cmnd(TSDAS_ADDA1_pfg_cmnd(PFG2_N_DATA,PFG2_M_DATA,PFG2_V_DATA,PFG2_X_DATA,PFG2_R_DATA,PFG2_Z_DATA));

		// now wait for SIO-0 transmitter is empty in order
		// to ensure that transmission has been completed prior proceeding
		// execution. Otherwise, switching to SYNC mode appear in the mid of
		// active transmission of command word.
	_WAIT_SIO_TEMPTY;

	adc_data_start_ptr	= data;
    adc_save_ptr		= data;
	dac_data_start_ptr	= data;
    dac_get_ptr			= data;

    dac_data_count_max	= _EPOCH_LENGTH;
    adc_data_count_max	= _EPOCH_LENGTH;
	dac_data_count		= 0;
	adc_data_count		= 0;


	// set SYNC mode
	init_sio_sync();

	// enable interrupts and finally start AD/DA acquisition...
	C3X_SR0_INT_ENABLE;
	C3X_SX0_INT_ENABLE;
    C3X_INT_ENABLE;

	// start communcation in SYNC mode
	set_sync_mode();

	while(1);	// loop forever

}

