/********************************************************************
   T/SDAS-AD1/14/300K-DA1/16/300K DCM rev.2A Demo Utility for C64xx DSP. rev.1A-1
   (C) MicroLAB Systems, 1999-2005


   Description:
   ------------
	1. This demo writes provides pass-through for A/D-D/A channel in SYNC mode.
        2. DCM on-board PFG is used to set sampling frequency.
	3. DSP on-chip timer TM-1 is used to set SYNC mode, and on-board configuration
	   swict must be set to S/W PROGRAMMABLE OPERATION MODE and XTIMER-1 line to
	   be used to set s/w programmable SYNC mode.


   TORNADO platform:
   -----------------
        This demo compiles for TORNADO-6x, TORNADO-P6x and TORNADO-E6x
	boards and actually differs in PIOX/SIOX reset control and
	initialization of EMIF registers to meet the board
	specific DSP memory map. This demo does not utilize any host
	PC ISA/PCI resources for TORNADO-6x/P6x boards and can run in
	stand-alone mode. MIRAGE JTAG emulator and TI CCS tools are
	required to recompile this demo and to upload code to the
	DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and is provided as it is
	without	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
     1. DCM on-board configuration switch:
		  - S/W PROGRAMMABLE OPERATION MODE must be selected
		  - XTIMER-1 line must be used to set operation mode 
	  	  - C6x McBSP-0 port is used (change this via _C6X_MCBSP_ def)
			
     2. TORNADO on-board configuration jumpers:
		  - TM0-OUT and TM1-OUT jumpers shall be installed
***********************************************************************/

#define	_E6X_		1
#define	_E64XX_		4
#define	_T6X_		2
#define	_TP6X_		3
#define	_TP64XX_	5

#define	_C6X_MCBSP0_	0
#define	_C6X_MCBSP1_	1

//~~~~~~~~~ TORNADO and C6x McBSP port selectors for compilation ~~~~~~~
//!!!! modify this def in order to compile for specific TORNADO platform
#define	_TORNADO_	_TP6X_

//!!!! modify this def in order to compile for McBSP-0 or McBSP-1 port
#define	_C6X_MCBSP_	_C6X_MCBSP0_
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define	_E6X_SELECTED		(_TORNADO_==_E6X_)
#define	_E64XX_SELECTED		(_TORNADO_==_E64XX_)
#define	_T6X_SELECTED		(_TORNADO_==_T6X_)
#define	_TP6X_SELECTED		(_TORNADO_==_TP6X_)
#define	_TP64XX_SELECTED	(_TORNADO_==_TP64XX_)

#define	_C6X_MCBSP0_SELECTED	(_C6X_MCBSP_==_C6X_MCBSP0_)
#define	_C6X_MCBSP1_SELECTED	(_C6X_MCBSP_==_C6X_MCBSP1_)

#if(_E64XX_SELECTED)
#define	__TE6X_SELECT_E64XX__	1
#endif

#if(_TP64XX_SELECTED)
#define	__TP6X_SELECT_TP64XX__	1
#endif

#include	"math.h"
#include	"c6x_def.h"
#include	"sxa1_dsp.h"

//------- macros for McBSP port access ----------------------
#if(_C6X_MCBSP0_SELECTED)	// define for C6x McBSP-0 port
#define	_WAIT_SIO_TRDY		while(!((*C6X_MCBSP0_SPCR) & 0x20000))
#define	_WAIT_SIO_TEMPTY	while( ((*C6X_MCBSP0_SPCR) & 0x40000))
#define	_WAIT_SIO_RRDY		while(!((*C6X_MCBSP0_SPCR) & 0x2))
#define	_WRITE_SIO_DATA		(*C6X_MCBSP0_DXR)
#define _READ_SIO_DATA		(*C6X_MCBSP0_DRR)
#endif

#if(_C6X_MCBSP1_SELECTED)	// define for C6x McBSP-1 port
#define	_WAIT_SIO_TRDY		while(!((*C6X_MCBSP1_SPCR) & 0x20000))
#define	_WAIT_SIO_TEMPTY	while( ((*C6X_MCBSP1_SPCR) & 0x40000))
#define	_WAIT_SIO_RRDY		while(!((*C6X_MCBSP1_SPCR) & 0x2))
#define	_WRITE_SIO_DATA		(*C6X_MCBSP1_DXR)
#define _READ_SIO_DATA		(*C6X_MCBSP1_DRR)
#endif


//------ TORNADO platform specific fucntions --------------------
#if(_E6X_SELECTED || _E64XX_SELECTED)
#define _REMOVE_SIOX_RESET	TE6X_set_pxsx_run(0x3)
#include	"te6x_dsp.h"
#endif

#if(_T6X_SELECTED)
#define _REMOVE_SIOX_RESET	T6X_set_pxsx_run(0x7)
#include	"t6x_dsp.h"
#endif

#if(_TP6X_SELECTED || _TP64XX_SELECTED)
#define _REMOVE_SIOX_RESET	TP6X_set_pxsx_run(0x7)
#include	"tp6x_dsp.h"
#endif
//--------------------------------------------------------------


		/* X/R interrupt handler parameters	*/

int	*dac_data_start_ptr,
        *dac_get_ptr,
	dac_data_count,
        dac_data_count_max;

int	*adc_save_ptr,
	*adc_data_start_ptr,
	adc_data_count,
	adc_data_count_max ;


	//-- SIO Transmitter interrupt.
	//   This IRQ is generated by after the transmitter gets empty after
	//   DCM has generated FSX and initialized transmission of DAC data.
	//   This IRQ is routed from McBSP-i XINT to INT4 C6x DSP interrupt request.

interrupt void xmit_int()
{
	//   write data to DAC from DATA array, which is filled in with 
	//   real-time ADC data.

	_WRITE_SIO_DATA = *dac_get_ptr;
        dac_get_ptr++;

        if((++dac_data_count) >=dac_data_count_max)
	 {       
		dac_data_count=0;
	        dac_get_ptr=dac_data_start_ptr;
	 }

}


	//-- McBSP port receiver interrupt.
	//   Read and save ADC data.
	//   This IRQ is routed from McBSP-i RINT to INT5 C6x DSP interrupt request.
int r_data;

interrupt void recv_int()
{
    r_data = _READ_SIO_DATA;

		// save ADC data in DATA array in particular ADC# subarray
    *adc_save_ptr = ((r_data <<16 )>>16);
    adc_save_ptr++;
    if((++adc_data_count) >=adc_data_count_max)
	 { 
		adc_data_count=0;
       	        adc_save_ptr=adc_data_start_ptr;
	 }
}



void init_cpu()
{
	// perform TORNADO-E6x/E64xx DSP System initialization
#if(_E6X_SELECTED || _E64XX_SELECTED)

		// init DSP EMIF
	TE6X_init_dsp_emif(TE6X_XMEM_SBSRAM);
#endif


	// perform TORNADO-6x DSP System initialization
#if(_T6X_SELECTED)

		// init DSP EMIF
	*C6X_EMIF_GCR=T6X_EMIF_GCR_DATA;
	*C6X_EMIF_CE0=T6X_EMIF_CE0_DATA;
	*C6X_EMIF_CE1=T6X_EMIF_CE1_DATA;
	*C6X_EMIF_CE2=T6X_EMIF_CE2_DATA;
	*C6X_EMIF_CE3=T6X_EMIF_CE3_DATA;
#endif


	// perform TORNADO-P6x DSP System initialization
#if(_TP6X_SELECTED)

		// init DSP EMIF
	TP6X_init_dsp_emif(TP6X_XMEM_SBSRAM);
#endif


	// perform TORNADO-P64xx DSP System initialization
#if(_TP64XX_SELECTED)

		// init DSP EMIF
	TP6X_init_dsp_emif();
#endif


	C6X_INT_CLEAR_ALL;
	C6X_INT_DISABLE_ALL;
	C6X_XINT_ALL_POLARITY_SET(C6X_XINT_POLARITY_NEG);

		// assume INT4    = McBSP-i Transmitter interrupt
		// assign INT5    = McBSP-i Receiver interrupt
#if(_C6X_MCBSP0_SELECTED)	
	C6X_INTMUX_SET(4,C6X_INTSEL_MCBSP0_XINT);
	C6X_INTMUX_SET(5,C6X_INTSEL_MCBSP0_RINT);
#endif
#if(_C6X_MCBSP1_SELECTED)	
	C6X_INTMUX_SET(4,C6X_INTSEL_MCBSP1_XINT);
	C6X_INTMUX_SET(5,C6X_INTSEL_MCBSP1_RINT);
#endif
}



void init_sio_async()
{
		//-- configure C6x McBSP port for ASYNC mode
#if(_C6X_MCBSP0_SELECTED)	// define for C6x McBSP-0 port
	*C6X_MCBSP0_SPCR = TSDAS_ADDA1_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP0_PCR  = TSDAS_ADDA1_C6X_MCBSP_PCR_DATA_ASYNC;
	*C6X_MCBSP0_XCR  = TSDAS_ADDA1_C6X_MCBSP_XCR_DATA_ASYNC;
	*C6X_MCBSP0_RCR  = TSDAS_ADDA1_C6X_MCBSP_RCR_DATA_ASYNC;
	*C6X_MCBSP0_SRGR = TSDAS_ADDA1_C6201_MCBSP_SRGR_DATA;
	*C6X_MCBSP0_SPCR |= TSDAS_ADDA1_C6X_MCBSP_SPCR_GO_MASK;
#endif

#if(_C6X_MCBSP1_SELECTED)	// define for C6x McBSP-1 port
	*C6X_MCBSP1_SPCR = TSDAS_ADDA1_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP1_PCR  = TSDAS_ADDA1_C6X_MCBSP_PCR_DATA_ASYNC;
	*C6X_MCBSP1_XCR  = TSDAS_ADDA1_C6X_MCBSP_XCR_DATA_ASYNC;
	*C6X_MCBSP1_RCR  = TSDAS_ADDA1_C6X_MCBSP_RCR_DATA_ASYNC;
	*C6X_MCBSP1_SRGR = TSDAS_ADDA1_C6201_MCBSP_SRGR_DATA;
	*C6X_MCBSP1_SPCR |= TSDAS_ADDA1_C6X_MCBSP_SPCR_GO_MASK;
#endif
}


void init_sio_sync()
{
		//-- configure C6x McBSP port for SYNC mode
#if(_C6X_MCBSP0_SELECTED)	// define for C6x McBSP-0 port
	*C6X_MCBSP0_SPCR = TSDAS_ADDA1_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP0_PCR  = TSDAS_ADDA1_C6X_MCBSP_PCR_DATA_SYNC;
	*C6X_MCBSP0_XCR  = TSDAS_ADDA1_C6X_MCBSP_XCR_DATA_SYNC;
	*C6X_MCBSP0_RCR  = TSDAS_ADDA1_C6X_MCBSP_RCR_DATA_SYNC;
	*C6X_MCBSP0_SRGR = TSDAS_ADDA1_C6201_MCBSP_SRGR_DATA;
	// start R/X
	*C6X_MCBSP0_SPCR |= TSDAS_ADDA1_C6X_MCBSP_SPCR_GO_MASK;
#endif

#if(_C6X_MCBSP1_SELECTED)	// define for C6x McBSP-1 port
	*C6X_MCBSP1_SPCR = TSDAS_ADDA1_C6X_MCBSP_SPCR_DATA;
	*C6X_MCBSP1_PCR  = TSDAS_ADDA1_C6X_MCBSP_PCR_DATA_SYNC;
	*C6X_MCBSP1_XCR  = TSDAS_ADDA1_C6X_MCBSP_XCR_DATA_SYNC;
	*C6X_MCBSP1_RCR  = TSDAS_ADDA1_C6X_MCBSP_RCR_DATA_SYNC;
	*C6X_MCBSP1_SRGR = TSDAS_ADDA1_C6201_MCBSP_SRGR_DATA;
	// start R/X
	*C6X_MCBSP1_SPCR |= TSDAS_ADDA1_C6X_MCBSP_SPCR_GO_MASK;
#endif

	// write false data to transmitter shift register in order 
	// these can be shifted out on the 1st FSX and prior interrupt occur
	_WRITE_SIO_DATA = 0;
}


	//--- write COMMAND DATA WORD in ASYNC mode
void write_async_cmnd(data)
unsigned data;
{
		//-- wait for McBSP transmitter is ready
	_WAIT_SIO_TRDY;

		//-- write command
	_WRITE_SIO_DATA = data;
}



	//-- configure TM-1 as OUTPUT pin for s/w programmable mode control
#define	_TM_CONTROL_RG_DATA_OUT0	0x00000000  // TM as OUT pin: '0' output state
#define	_TM_CONTROL_RG_DATA_OUT1	0x00000004  // TM as OUT pin: '1' output state

void set_async_mode()
{
		//-- configure TM-1 as OUTPUT pin
	*C6X_TM1_CONTROL  = _TM_CONTROL_RG_DATA_OUT0;	
}

void set_sync_mode()
{
		//-- configure TM-1 as OUTPUT pin
	*C6X_TM1_CONTROL  = _TM_CONTROL_RG_DATA_OUT1;	
}





#define	_EPOCH_LENGTH	20

// ADC/DAC data array
int data[_EPOCH_LENGTH];


// PFG programming
	// PFG_out=(10.0MHz*N*V)/(M*R*X*Z)
	// set PFG = 50 kHz 
	// (N=32, M=25, V=8, X=1, R=8, Z=256)
#define PFG2_N_DATA       32
#define PFG2_M_DATA       25
#define PFG2_V_DATA       TSDAS_ADDA1_PFG_V_X8
#define PFG2_X_DATA       TSDAS_ADDA1_PFG_X_DIV1
#define PFG2_R_DATA       TSDAS_ADDA1_PFG_R_DIV8
#define PFG2_Z_DATA       TSDAS_ADDA1_PFG_Z_DIV256



void main()
{  
	init_cpu();

	// first set ASYNC mode in order to program Fs SELECTOR and PFG
	set_async_mode();
	init_sio_async();

	// release SIOX reset control and enable DCM
	_REMOVE_SIOX_RESET;

	// set Fs selector to on-board PFG
	write_async_cmnd(TSDAS_ADDA1_CNTR_RG_WR_CMND | TSDAS_ADDA1_FSSEL_PFG);

	// configure PFG
	write_async_cmnd(TSDAS_ADDA1_pfg_cmnd(PFG2_N_DATA,PFG2_M_DATA,PFG2_V_DATA,PFG2_X_DATA,PFG2_R_DATA,PFG2_Z_DATA));

		// now wait for SIO-0 transmitter is empty in order
		// to ensure that transmission has been completed prior proceeding
		// execution. Otherwise, switching to SYNC mode appear in the mid of
		// active transmission of command word.
	_WAIT_SIO_TEMPTY;

	adc_data_start_ptr	= data;
        adc_save_ptr		= data;
	dac_data_start_ptr	= data;
        dac_get_ptr		= data;

        dac_data_count_max	= _EPOCH_LENGTH;
        adc_data_count_max	= _EPOCH_LENGTH;
	dac_data_count		= 0;
	adc_data_count		= 0;


	// set SYNC mode
	init_sio_sync();


	// enable interrupts and finally start AD/DA acquisition...
	C6X_INT_ENABLE(4);
	C6X_INT_ENABLE(5);
	C6X_GINT_ENABLE;

	// start communcation in SYNC mode
	set_sync_mode();

	while(1);	// loop forever

}